@@ -1,5 +1,5 @@
 package CPAN::Testers::API::Controller::Report;
-our $VERSION = '0.007';
+our $VERSION = '0.008';
 # ABSTRACT: Work with raw test reports
 
 #pod =head1 DESCRIPTION
@@ -15,8 +15,6 @@ our $VERSION = '0.007';
 
 use Mojo::Base 'Mojolicious::Controller';
 use CPAN::Testers::API::Base;
-use Mojo::JSON qw( encode_json );
-use Data::UUID;
 
 #pod =method report_post
 #pod
@@ -36,19 +34,52 @@ use Data::UUID;
 sub report_post( $c ) {
     $c->openapi->valid_input or return;
     my $report = $c->validation->param( 'report' );
-    $report->{id} = Data::UUID->new->create_str();
-    $c->schema->resultset( 'TestReport' )->create( {
-        id => $report->{id},
-        report => encode_json( $report ),
+    my $row = $c->schema->resultset( 'TestReport' )->create( {
+        report => $report,
     } );
     return $c->render(
         status => 201,
         openapi => {
-            id => $report->{id},
+            id => $row->id,
         },
     );
 }
 
+#pod =method report_get
+#pod
+#pod     ### Requests:
+#pod     GET /v3/report/:guid
+#pod
+#pod     ### Response
+#pod     200 OK
+#pod     { "id": "...", ... }
+#pod
+#pod Get a single CPAN Testers report from the database.
+#pod
+#pod =cut
+
+sub report_get( $c ) {
+    $c->openapi->valid_input or return;
+    my $id = $c->validation->param( 'id' );
+    my $row = $c->schema->resultset( 'TestReport' )->find( $id );
+    if ( !$row ) {
+        return $c->render(
+            status => 404,
+            openapi => {
+                errors => [
+                    {
+                        message => 'Report ID not found',
+                        path => '/id',
+                    },
+                ],
+            },
+        );
+    }
+    return $c->render(
+        openapi => $row->report,
+    );
+}
+
 1;
 
 __END__
@@ -61,7 +92,7 @@ CPAN::Testers::API::Controller::Report - Work with raw test reports
 
 =head1 VERSION
 
-version 0.007
+version 0.008
 
 =head1 DESCRIPTION
 
@@ -83,6 +114,17 @@ submitted by the army of testers of CPAN.
 Submit a new CPAN Testers report. This is used by testers when they're
 finished running a test.
 
+=head2 report_get
+
+    ### Requests:
+    GET /v3/report/:guid
+
+    ### Response
+    200 OK
+    { "id": "...", ... }
+
+Get a single CPAN Testers report from the database.
+
 =head1 SEE ALSO
 
 L<CPAN::Testers::Schema::Result::TestReport>, L<Mojolicious::Controller>
